@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_auth.c,v 1.5 2004/04/28 13:52:38 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_auth.c,v 1.6 2004/05/06 06:33:58 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -36,15 +36,15 @@
 #include <errno.h>
 #include <string.h>
 
-static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *frontend);
-static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend);
-static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend);
+static POOL_STATUS pool_send_auth_ok(POOL_CONNECTION *frontend, int pid, int key);
+static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reauth);
+static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reauth);
+static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reauth);
 
 /*
 * do authentication against backend. if success return 0 otherwise non 0.
 */
-POOL_STATUS pool_do_auth(POOL_CONNECTION *frontend,
-						 POOL_CONNECTION_POOL *cp)
+POOL_STATUS pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 {
 	int status;
 	char kind, kind1;
@@ -70,7 +70,7 @@ POOL_STATUS pool_do_auth(POOL_CONNECTION *frontend,
 
 		if (kind != kind1)
 		{
-			pool_error("pool_do_auth: kind is not match between backends master(%c) secondary(%c)",
+			pool_error("pool_do_auth: kind does not match between backends master(%c) secondary(%c)",
 					   kind, kind1);
 			return -1;
 		}
@@ -124,16 +124,22 @@ POOL_STATUS pool_do_auth(POOL_CONNECTION *frontend,
 
 	pid = ntohl(pid);
 
+	/* trust? */
+	if (pid == 0)
+	{
+		MASTER_CONNECTION(cp)->con->auth_kind = 0;
+	}
+
 	/* clear text password authentication? */
-	if (pid == 3)
+	else if (pid == 3)
 	{
 		pool_debug("trying clear text password authentication");
 
-		pid = do_clear_text_password(MASTER_CONNECTION(cp)->con, frontend);
+		pid = do_clear_text_password(MASTER_CONNECTION(cp)->con, frontend, 0);
 
 		if (pid >= 0 && pool_config.replication_enabled)
 		{
-			pid = do_clear_text_password(SECONDARY_CONNECTION(cp)->con, frontend);
+			pid = do_clear_text_password(SECONDARY_CONNECTION(cp)->con, frontend, 0);
 		}
 	}
 
@@ -142,11 +148,11 @@ POOL_STATUS pool_do_auth(POOL_CONNECTION *frontend,
 	{
 		pool_debug("trying crypt authentication");
 
-		pid = do_crypt(MASTER_CONNECTION(cp)->con, frontend);
+		pid = do_crypt(MASTER_CONNECTION(cp)->con, frontend, 0);
 
 		if (pid >= 0 && pool_config.replication_enabled)
 		{
-			pid = do_crypt(SECONDARY_CONNECTION(cp)->con, frontend);
+			pid = do_crypt(SECONDARY_CONNECTION(cp)->con, frontend, 0);
 		}
 	}
 
@@ -155,11 +161,11 @@ POOL_STATUS pool_do_auth(POOL_CONNECTION *frontend,
 	{
 		pool_debug("trying MD5 authentication");
 
-		pid = do_md5(MASTER_CONNECTION(cp)->con, frontend);
+		pid = do_md5(MASTER_CONNECTION(cp)->con, frontend, 0);
 
 		if (pid >= 0 && pool_config.replication_enabled)
 		{
-			pid = do_md5(SECONDARY_CONNECTION(cp)->con, frontend);
+			pid = do_md5(SECONDARY_CONNECTION(cp)->con, frontend, 0);
 		}
 	}
 
@@ -233,11 +239,44 @@ POOL_STATUS pool_do_auth(POOL_CONNECTION *frontend,
 	return (pool_send_auth_ok(frontend, pid, key));
 }
 
+/*
+* do re-authentication for reused connection. if success return 0 otherwise non 0.
+*/
+POOL_STATUS pool_do_reauth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
+{
+	int status;
+
+	/* trust? */
+	if (MASTER_CONNECTION(cp)->con->auth_kind == 0)
+		status = 0;
+
+	/* clear text password? */
+	else if (MASTER_CONNECTION(cp)->con->auth_kind == 3)
+	{
+		status = do_clear_text_password(MASTER_CONNECTION(cp)->con, frontend, 1);
+	}
+
+	/* crypt password? */
+	else if (MASTER_CONNECTION(cp)->con->auth_kind == 4)
+	{
+		status = do_crypt(MASTER_CONNECTION(cp)->con, frontend, 1);
+	}
+
+	/* md5 password? */
+	else if (MASTER_CONNECTION(cp)->con->auth_kind == 5)
+	{
+		status = do_md5(MASTER_CONNECTION(cp)->con, frontend, 1);
+	}
+
+	if (status == 0)
+		return (pool_send_auth_ok(frontend, MASTER_CONNECTION(cp)->pid, MASTER_CONNECTION(cp)->key) != POOL_CONTINUE);
+	return -1;
+}
 
 /*
 * send authentication ok to frontend. if success return 0 otherwise non 0.
 */
-POOL_STATUS pool_send_auth_ok(POOL_CONNECTION *frontend, int pid, int key)
+static POOL_STATUS pool_send_auth_ok(POOL_CONNECTION *frontend, int pid, int key)
 {
 	char kind;
 	char buff[128];
@@ -268,10 +307,10 @@ POOL_STATUS pool_send_auth_ok(POOL_CONNECTION *frontend, int pid, int key)
 /*
  * perform clear text password authetication
  */
-static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
+static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reauth)
 {
 	static int size;
-	static char password[1024];
+	static char password[MAX_PASSWORD_SIZE];
 	char response;
 	int kind;
 
@@ -289,6 +328,12 @@ static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *fro
 			return -1;
 		}
 
+		if ((ntohl(size) - 4) > sizeof(password))
+		{
+			pool_error("do_clear_text_password: password is too long(size: %d)", ntohl(size) - 4);
+			return -1;
+		}
+
 		if (pool_read(frontend, password, ntohl(size) - 4))
 		{
 			pool_error("do_clear_text_password: failed to read password (size: %d)", ntohl(size) - 4);
@@ -296,6 +341,24 @@ static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *fro
 		}
 	}
 
+	/* connection reusing? */
+	if (reauth)
+	{
+		if ((ntohl(size) - 4) != backend->pwd_size)
+		{
+			pool_debug("do_clear_text_password; password size does not match in re-authetication");
+			return -1;
+		}
+
+		if (memcmp(password, backend->password, backend->pwd_size) != 0)
+		{
+			pool_debug("do_clear_text_password; password does not match in re-authetication");
+			return -1;
+		}
+
+		return 0;
+	}
+
 	/* send password packet to backend */
 	pool_write(backend, &size, sizeof(size));
 	pool_write_and_flush(backend, password, ntohl(size) -4);
@@ -318,25 +381,39 @@ static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *fro
 		return -1;
 	}
 
+	/* if authenticated, save info */
+	if (!reauth && kind == 0)
+	{
+		backend->auth_kind = 3;
+		backend->pwd_size = ntohl(size) - 4;
+		memcpy(backend->password, password, backend->pwd_size);
+	}
 	return kind;
 }
 
 /*
  * perform crypt authetication
  */
-static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
+static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reauth)
 {
 	char salt[2];
 	static int size;
-	static char password[1024];
+	static char password[MAX_PASSWORD_SIZE];
 	char response;
 	int kind;
 
-	/* read salt */
-	if (pool_read(backend, salt, sizeof(salt)))
+	if (!reauth)
 	{
-		pool_error("do_crypt: failed to read salt");
-		return -1;
+		/* read salt */
+		if (pool_read(backend, salt, sizeof(salt)))
+		{
+			pool_error("do_crypt: failed to read salt");
+			return -1;
+		}
+	}
+	else
+	{
+		memcpy(salt, backend->salt, sizeof(salt));
 	}
 
 	/* master? */
@@ -354,6 +431,12 @@ static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
 			return -1;
 		}
 
+		if ((ntohl(size) - 4) > sizeof(password))
+		{
+			pool_error("do_crypt: password is too long(size: %d)", ntohl(size) - 4);
+			return -1;
+		}
+
 		if (pool_read(frontend, password, ntohl(size) - 4))
 		{
 			pool_error("do_crypt: failed to read password (size: %d)", ntohl(size) - 4);
@@ -361,6 +444,25 @@ static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
 		}
 	}
 
+	/* connection reusing? */
+	if (reauth)
+	{
+		pool_debug("size: %d saved_size: %d", (ntohl(size) - 4), backend->pwd_size);
+		if ((ntohl(size) - 4) != backend->pwd_size)
+		{
+			pool_debug("do_crypt: password size does not match in re-authetication");
+			return -1;
+		}
+
+		if (memcmp(password, backend->password, backend->pwd_size) != 0)
+		{
+			pool_debug("do_crypt: password does not match in re-authetication");
+			return -1;
+		}
+
+		return 0;
+	}
+
 	/* send password packet to backend */
 	pool_write(backend, &size, sizeof(size));
 	pool_write_and_flush(backend, password, ntohl(size) -4);
@@ -383,17 +485,25 @@ static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
 		return -1;
 	}
 
+	/* if authenticated, save info */
+	if (kind == 0)
+	{
+		backend->auth_kind = 4;
+		backend->pwd_size = ntohl(size) - 4;
+		memcpy(backend->password, password, backend->pwd_size);
+		memcpy(backend->salt, salt, sizeof(salt));
+	}
 	return kind;
 }
 
 /*
  * perform MD5 authetication
  */
-static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
+static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reauth)
 {
 	char salt[4];
 	static int size;
-	static char password[1024];
+	static char password[MAX_PASSWORD_SIZE];
 	char response;
 	int kind;
 
@@ -403,6 +513,10 @@ static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
 		pool_error("do_md5: failed to read salt");
 		return -1;
 	}
+	else
+	{
+		memcpy(salt, backend->salt, sizeof(salt));
+	}
 
 	/* master? */
 	if (!backend->issecondary_backend)
@@ -419,6 +533,12 @@ static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
 			return -1;
 		}
 
+		if ((ntohl(size) - 4) > sizeof(password))
+		{
+			pool_error("do_md5: password is too long(size: %d)", ntohl(size) - 4);
+			return -1;
+		}
+
 		if (pool_read(frontend, password, ntohl(size) - 4))
 		{
 			pool_error("do_md5: failed to read password (size: %d)", ntohl(size) - 4);
@@ -426,6 +546,24 @@ static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
 		}
 	}
 
+	/* connection reusing? */
+	if (reauth)
+	{
+		if ((ntohl(size) - 4) != backend->pwd_size)
+		{
+			pool_debug("do_md5; password size does not match in re-authetication");
+			return -1;
+		}
+
+		if (memcmp(password, backend->password, backend->pwd_size) != 0)
+		{
+			pool_debug("do_md5; password does not match in re-authetication");
+			return -1;
+		}
+
+		return 0;
+	}
+
 	/* send password packet to backend */
 	pool_write(backend, &size, sizeof(size));
 	pool_write_and_flush(backend, password, ntohl(size) -4);
@@ -448,5 +586,13 @@ static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend)
 		return -1;
 	}
 
+	/* if authenticated, save info */
+	if (kind == 0)
+	{
+		backend->auth_kind = 5;
+		backend->pwd_size = ntohl(size) - 4;
+		memcpy(backend->password, password, backend->pwd_size);
+		memcpy(backend->salt, salt, sizeof(salt));
+	}
 	return kind;
 }
