@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.16 2004/04/28 13:52:38 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.17 2004/05/06 06:37:02 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -211,6 +211,8 @@ RETSIGTYPE pool_backend_timer_handler(int sig)
 
 	for (i=0;i<pool_config.max_pool;i++, p++)
 	{
+		if (!MASTER_CONNECTION(p))
+			continue;
 		if (MASTER_CONNECTION(p)->user == NULL)
 			continue;
 
@@ -224,10 +226,21 @@ RETSIGTYPE pool_backend_timer_handler(int sig)
 			{
 				/* discard expired connection */
 				pool_debug("pool_backend_timer_handler: expires user %s database %s", MASTER_CONNECTION(p)->user, MASTER_CONNECTION(p)->database);
+
+				pool_send_frontend_exits(p);
+
 				free(MASTER_CONNECTION(p)->user);
 				free(MASTER_CONNECTION(p)->database);
 				pool_close(MASTER_CONNECTION(p)->con);
 				memset(p, 0, sizeof(POOL_CONNECTION_POOL));
+
+				if (pool_config.replication_enabled)
+				{
+					free(SECONDARY_CONNECTION(p)->user);
+					free(SECONDARY_CONNECTION(p)->database);
+					pool_close(SECONDARY_CONNECTION(p)->con);
+					memset(p, 0, sizeof(POOL_CONNECTION_POOL));
+				}
 			}
 			else
 			{
