@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.15 2004/05/07 13:57:52 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.16 2004/05/11 09:27:14 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -418,7 +418,7 @@ static POOL_STATUS CompleteCommandResponse(POOL_CONNECTION *frontend,
 	/* forward to the frontend */
 	pool_write(frontend, "C", 1);
 	pool_debug("Complete Command Response: string: \"%s\"", string);
-	if (pool_write_and_flush(frontend, string, len) < 0)
+	if (pool_write(frontend, string, len) < 0)
 	{
 		return POOL_END;
 	}
@@ -520,10 +520,12 @@ static int RowDescription(POOL_CONNECTION *frontend,
 		pool_write(frontend, &mod, sizeof(int));
 	}
 
+#ifdef NOT_USED
 	if (pool_flush(frontend) < 0)
 	{
 		return POOL_END;
 	}
+#endif
 
 	return num_fields;
 }
@@ -635,10 +637,12 @@ static POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 		mask >>= 1;
 	}
 
+#ifdef NOT_USED
 	if (pool_flush(frontend) < 0)
 	{
 		return POOL_END;
 	}
+#endif
 
 	return POOL_CONTINUE;
 }
@@ -737,7 +741,7 @@ static POOL_STATUS CursorResponse(POOL_CONNECTION *frontend,
 
 	/* forward to the frontend */
 	pool_write(frontend, "P", 1);
-	if (pool_write_and_flush(frontend, string, len) < 0)
+	if (pool_write(frontend, string, len) < 0)
 	{
 		return POOL_END;
 	}
