@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
-* $Header: /home/t-ishii/repository/pgpool/pool_stream.c,v 1.17 2004/05/07 13:59:23 t-ishii Exp $
+* $Header: /home/t-ishii/repository/pgpool/pool_stream.c,v 1.18 2004/05/11 09:27:14 t-ishii Exp $
 *
 * pgpool: a language independent connection pool server for PostgreSQL 
 * written by Tatsuo Ishii
@@ -36,6 +36,8 @@
 
 static int mystrlen(char *str, int upper, int *flag);
 static int mystrlinelen(char *str, int upper, int *flag);
+static int save_pending_data(POOL_CONNECTION *cp, void *data, int len);
+static int consume_pending_data(POOL_CONNECTION *cp, void *data, int len);
 
 /*
 * open read/write file descriptors.
@@ -54,6 +56,7 @@ POOL_CONNECTION *pool_open(int fd)
 
 	memset(cp, 0, sizeof(*cp));
 
+#ifdef NOT_USED
 	cp->read_fd = fdopen(fd, "r");
 	if (cp->read_fd == NULL)
 	{
@@ -61,12 +64,15 @@ POOL_CONNECTION *pool_open(int fd)
 		free(cp);
 		return NULL;
 	}
+#endif
 
 	cp->write_fd = fdopen(fd, "w");
 	if (cp->write_fd == NULL)
 	{
 		pool_error("pool_open: fdopen failed: %s",strerror(errno));
+#ifdef NOT_USED
 		fclose(cp->read_fd);
+#endif
 		free(cp);
 		return NULL;
 	}
@@ -93,9 +99,11 @@ POOL_CONNECTION *pool_open(int fd)
 */
 void pool_close(POOL_CONNECTION *cp)
 {
-	fclose(cp->read_fd);
 	fclose(cp->write_fd);
+#ifdef NOT_USED
+	fclose(cp->read_fd);
 	close(cp->fd);
+#endif
 	free(cp->hp);
 	free(cp->sbuf);
 	free(cp);
@@ -107,27 +115,14 @@ void pool_close(POOL_CONNECTION *cp)
 */
 int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 {
+	static char readbuf[READBUFSZ];
+
+	int consume_size;
 	int readlen;
 
-	if (cp->len)
-	{
-		/* consume pending data */
-		if (cp->len >= len)
-		{
-			/* there is enough pending data */
-			memmove(buf, cp->hp+cp->po, len);
-			cp->po += len;
-			cp->len -= len;
-			return 0;
-		}
-		else
-		{
-			memmove(buf, cp->hp+cp->po, cp->len);
-			len -= cp->len;
-			buf += cp->len;
-			cp->len = 0;
-		}
-	}
+	consume_size = consume_pending_data(cp, buf, len);
+	len -= consume_size;
+	buf += consume_size;
 
 	while (len > 0)
 	{
@@ -140,7 +135,7 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 			}
 		}
 
-		readlen = read(cp->fd, buf, len);
+		readlen = read(cp->fd, readbuf, READBUFSZ);
 		if (readlen == -1)
 		{
 			pool_error("pool_read: read failed (%s)", strerror(errno));
@@ -174,16 +169,19 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 				return -1;
 			}
 		}
-		else if (len > readlen)
-		{
-			pool_debug("pool_read: expected %d bytes got %d retrying...", len, readlen);
-			buf += readlen;
-			len -= readlen;
-		}
-		else
+
+		if (len < readlen)
 		{
+			/* overrun. we need to save remaining data to pending buffer */
+			if (save_pending_data(cp, readbuf+len, readlen-len))
+				return -1;
+			memmove(buf, readbuf, len);
 			break;
 		}
+
+		memmove(buf, readbuf, readlen);
+		buf += readlen;
+		len -= readlen;
 	}
 
 #ifdef DEBUG
@@ -250,6 +248,7 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 	int readlen;
 	int strlength;
 	int flag;
+	int consume_size;
 
 	int secondary = cp->issecondary_backend;
 
@@ -293,29 +292,24 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 			}
 		}
 
-		/* copy pending data to buffer */
-		memmove(cp->sbuf, cp->hp+cp->po, strlength);
+		/* consume pending and save to read string buffer */
+		consume_size = consume_pending_data(cp, cp->sbuf, strlength);
 
-#ifdef DEBUG
-		snprintf(pbuf, strlength, buf);
-
-		pool_debug("copy partial pending data :%s: po: %d remain: %d len: %d", pbuf, cp->po, cp->len, strlength);
-#endif
-		cp->po += strlength;
-		cp->len -= strlength;
 		*len = strlength;
 
 		/* is the string null terminated? */
-		if (cp->len <= 0 && !flag)
+		if (consume_size == strlength && !flag)
 		{
-			/* not null or line terminated */
-			cp->len = 0;
-			/* we need to read more since we have not encountered NULL yet */
+			/* not null or line terminated.
+			 * we need to read more since we have not encountered NULL or new line yet
+			 */
 			readsize = cp->sbufsz - strlength;
 			readp = strlength;
 		}
 		else
 		{
+			pool_debug("pool_read_string: read all from pending data. po:%d len:%d",
+					   cp->po, cp->len);
 			return cp->sbuf;
 		}
 	} else
@@ -326,8 +320,6 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 
 	for (;;)
 	{
-		pool_debug("pool_read_string: readsize: %d readp: %d secondary: %d", readsize, readp, secondary);
-
 		readlen = read(cp->fd, cp->sbuf+readp, readsize);
 		if (readlen == -1)
 		{
@@ -344,61 +336,30 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 			}
 		}
 
-		pool_debug("pool_read_string: buf:%s: readlen: %d", cp->sbuf, readlen);
-
 		/* check overrun */
 		if (line)
 			strlength = mystrlinelen(cp->sbuf+readp, readlen, &flag);
 		else
 			strlength = mystrlen(cp->sbuf+readp, readlen, &flag);
 
-		pool_debug("strlength: %d", strlength);
-
 		if (strlength < readlen)
 		{
-			/* save pending data */
-			cp->len = readlen - strlength;
-
-			/* pending buffer is enough? */
-			if (cp->len > cp->bufsz)
-			{
-				/* too small, enlarge it. we assume that pending data
-                   size is never larger than cp->sbufsz */
-
-				pool_debug("pool_read_string: enlarge pending buffer. old: %d new: %d", cp->bufsz, cp->sbufsz);
-
-				cp->hp = realloc(cp->hp, cp->sbufsz);
-				if (cp->hp == NULL)
-				{
-					pool_error("pool_read_string: realloc failed");
-					return NULL;
-				}
-			}
-
-			memmove(cp->hp, cp->sbuf+readp+strlength, cp->len);
-			cp->po = 0;
+			save_pending_data(cp, cp->sbuf+readp+strlength, readlen-strlength);
 			*len += strlength;
-			pool_debug("pending data saved: :%s: length: %d return data: :%s: length: %d", cp->hp, cp->len, cp->sbuf, *len);
-
+			pool_debug("pool_read_string: total result %d with pending data po:%d len:%d", *len, cp->po, cp->len);
 			return cp->sbuf;
 		}
 
 		*len += readlen;
 
-		pool_debug("pool_read_string: string: :%s:", cp->sbuf+readp);
-		pool_debug("pool_read_string: next string: :%s:", cp->sbuf+readp+strlen(cp->sbuf+readp)+1);
-		
 		/* encountered null or newline? */
 		if (flag)
 		{
 			/* ok we have read all data */
-			pool_debug("pool_read_string: final read %d string: :%s:", readlen, cp->sbuf+readp);
-			pool_debug("pool_read_string: total result %d string: :%s:", *len, cp->sbuf);
+			pool_debug("pool_read_string: total result %d ", *len);
 			break;
 		}
 
-		pool_debug("pool_read_string: read more data %d string: :%s:", *len, cp->sbuf+readp);
-
 		readp += readlen;
 		readsize = READBUFSZ;
 
@@ -417,7 +378,6 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 	return cp->sbuf;
 }
 
-
 /*
  * returns the byte length of str, including \0, no more than upper.
  * if encountered \0, flag is set to non 0.
@@ -455,7 +415,6 @@ static int mystrlen(char *str, int upper, int *flag)
  *	mystrlinelen("abc", 5) returns 4
  *	mystrlinelen("abcd\nefg", 4) returns 4
  *	mystrlinelen("abcd\nefg", 5) returns 5
- *	mystrlinelen("abcd\nefg", 5) returns 5
  *	mystrlinelen("abcd\nefg", 6) returns 5
  */
 static int mystrlinelen(char *str, int upper, int *flag)
@@ -475,3 +434,62 @@ static int mystrlinelen(char *str, int upper, int *flag)
 	}
 	return len;
 }
+
+/*
+ * save pending data
+ */
+static int save_pending_data(POOL_CONNECTION *cp, void *data, int len)
+{
+	int reqlen;
+	size_t realloc_size;
+	char *p;
+
+	/* to be safe */
+	if (cp->len == 0)
+		cp->po = 0;
+
+	reqlen = cp->po + cp->len + len;
+
+	/* pending buffer is enough? */
+	if (reqlen > cp->bufsz)
+	{
+		/* too small, enlarge it */
+		realloc_size = (reqlen/READBUFSZ+1)*READBUFSZ;
+		p = realloc(cp->hp, realloc_size);
+		if (p == NULL)
+		{
+			pool_error("save_pending_data: realloc failed");
+			return -1;
+		}
+
+		cp->bufsz = realloc_size;
+		cp->hp = p;
+	}
+
+	memmove(cp->hp + cp->po + cp->len, data, len);
+	cp->len += len;
+
+	return 0;
+}
+
+/*
+ * consume pending data. returns actually consumed data length.
+ */
+static int consume_pending_data(POOL_CONNECTION *cp, void *data, int len)
+{
+	int consume_size;
+
+	if (cp->len <= 0)
+		return 0;
+
+	consume_size = Min(len, cp->len);
+	memmove(data, cp->hp + cp->po, consume_size);
+	cp->len -= consume_size;
+
+	if (cp->len <= 0)
+		cp->po = 0;
+	else
+		cp->po += consume_size;
+
+	return consume_size;
+}
